@@ -7,6 +7,7 @@
 #include "winctrls.h"
 
 #include "winpriv.h"
+#include "charset.h"  // wcscpy
 
 #define _RPCNDR_H
 #define _WTYPES_H
@@ -782,8 +783,7 @@ winctrl_layout(winctrls *wc, ctrlpos *cp, controlset *s, int *id)
 
     if (colstart >= 0) {
      /*
-      * Update the ypos in all columns crossed by this
-      * control.
+      * Update the ypos in all columns crossed by this control.
       */
       int i;
       for (i = colstart; i < colstart + colspan; i++)
@@ -814,37 +814,122 @@ winctrl_set_focus(control *ctrl, int has_focus)
     dlg.focused = null;
 }
 
+#ifndef CF_INACTIVEFONTS
+# ifdef __MSABI_LONG
+#define CF_INACTIVEFONTS __MSABI_LONG (0x02000000)
+# else
+#define CF_INACTIVEFONTS 0x02000000
+# endif
+#endif
+
+#define dont_debug_fontsel
+
+#ifdef debug_fontsel
+#define trace_fontsel(params)	printf params
+#else
+#define trace_fontsel(params)	
+#endif
+
 static void
 select_font(winctrl *c)
 {
   font_spec fs = *(font_spec *) c->data;
+  LOGFONTW lf;
   HDC dc = GetDC(0);
-  LOGFONT lf;
   lf.lfHeight = -MulDiv(fs.size, GetDeviceCaps(dc, LOGPIXELSY), 72);
   ReleaseDC(0, dc);
   lf.lfWidth = lf.lfEscapement = lf.lfOrientation = 0;
   lf.lfItalic = lf.lfUnderline = lf.lfStrikeOut = 0;
-  lf.lfWeight = (fs.isbold ? FW_BOLD : 0);
+  lf.lfWeight = fs.weight ? fs.weight : fs.isbold ? FW_BOLD : 0;
   lf.lfCharSet = DEFAULT_CHARSET;
   lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
   lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
   lf.lfQuality = DEFAULT_QUALITY;
   lf.lfPitchAndFamily = FIXED_PITCH | FF_DONTCARE;
-  strlcpy(lf.lfFaceName, fs.name, sizeof lf.lfFaceName);
+#if CYGWIN_VERSION_API_MINOR >= 201
+  swprintf(lf.lfFaceName, lengthof(lf.lfFaceName), L"%ls", fs.name);
+#else
+  if (wcslen(fs.name) < lengthof(lf.lfFaceName))
+    wcscpy(lf.lfFaceName, fs.name);
+  else
+    wcscpy(lf.lfFaceName, L"Lucida Console");
+#endif
 
-  CHOOSEFONT cf;
+  UINT_PTR CALLBACK fonthook(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
+  {
+    (void)lParam;
+    if (uiMsg == WM_COMMAND && wParam == 1026) {  // Apply
+      LOGFONTW lfapply;
+      SendMessageW(hdlg, WM_CHOOSEFONT_GETLOGFONT, 0, (LPARAM)&lfapply);
+      font_spec * fsp = &new_cfg.font;
+      wstrset(&fsp->name, lfapply.lfFaceName);
+      trace_fontsel(("apply <%ls>\n", lfapply.lfFaceName));
+      HDC dc = GetDC(0);
+      fsp->size = -MulDiv(lfapply.lfHeight, 72, GetDeviceCaps(dc, LOGPIXELSY));
+      ReleaseDC(0, dc);
+      fsp->weight = lfapply.lfWeight;
+      fsp->isbold = (lfapply.lfWeight >= FW_BOLD);
+      // apply font
+      apply_config(false);
+      // update font spec label
+      c->ctrl->handler(c->ctrl, EVENT_REFRESH);  // -> dlg_stdfontsel_handler
+      //or dlg_fontsel_set(c->ctrl, fsp);
+    }
+    else if (uiMsg == WM_COMMAND && wParam == 1) {  // OK
+#ifdef failed_workaround_for_no_font_issue
+      /*
+        Trying to work-around issue #507
+        "There is no font with that name."
+        "Choose a font from the list of fonts."
+        as it occurs with Meslo LG L DZ for Powerline
+      */
+      LOGFONTW lfapply;
+      SendMessageW(hdlg, WM_CHOOSEFONT_GETLOGFONT, 0, (LPARAM)&lfapply);
+      // lfapply.lfFaceName is "Meslo LG L DZ for Powerline"
+      HWND wnd = GetDlgItem(hdlg, c->base_id +99);
+      int size = GetWindowTextLengthW(wnd) + 1;
+      wchar * fn = newn(wchar, size);
+      GetWindowTextW(wnd, fn, size);
+      // fn is "Meslo LG L DZ for Powerline RegularForPowerline"
+      // trying to fix the inconsistency with
+      SetWindowTextW(wnd, lfapply.lfFaceName);
+      // does not help...
+      // what a crap!
+#endif
+    }
+    else if (uiMsg == WM_COMMAND && wParam == 2) {  // Cancel
+    }
+    return 0;  // default processing
+  }
+
+  CHOOSEFONTW cf;
   cf.lStructSize = sizeof (cf);
   cf.hwndOwner = dlg.wnd;
   cf.lpLogFont = &lf;
+  cf.lpfnHook = fonthook;
   cf.Flags =
-    CF_FIXEDPITCHONLY | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT |
-    CF_SCREENFONTS | CF_NOSCRIPTSEL;
+    CF_INITTOLOGFONTSTRUCT | CF_FORCEFONTEXIST
+    | CF_FIXEDPITCHONLY | CF_NOVERTFONTS
+    | CF_NOSCRIPTSEL
+    | CF_SCRIPTSONLY    // exclude fonts with OEM or SYMBOL charset range
+    | CF_APPLY | CF_ENABLEHOOK  // enable Apply button
+    ;
+  if (new_cfg.show_hidden_fonts)
+    // include fonts marked to Hide in Fonts Control Panel
+    cf.Flags |= CF_INACTIVEFONTS;
 
-  if (ChooseFont(&cf)) {
-    strset(&fs.name, lf.lfFaceName);
-    fs.isbold = (lf.lfWeight == FW_BOLD);
+  // open font selection menu
+  if (ChooseFontW(&cf)) {
+    // font selection menu closed with OK
+    trace_fontsel(("OK <%ls>\n", lf.lfFaceName));
+    wstrset(&fs.name, lf.lfFaceName);
+    // here we could enumerate font widths and adjust...
+    // rather doing that in win_init_fonts
     fs.size = cf.iPointSize / 10;
+    fs.weight = lf.lfWeight;
+    fs.isbold = (lf.lfWeight >= FW_BOLD);
     dlg_fontsel_set(c->ctrl, &fs);
+    //call dlg_stdfontsel_handler
     c->ctrl->handler(c->ctrl, EVENT_VALCHANGE);
   }
 }
@@ -1203,16 +1288,49 @@ dlg_fontsel_set(control *ctrl, font_spec *fs)
   winctrl *c = ctrl->plat_ctrl;
   assert(c && c->ctrl->type == CTRL_FONTSELECT);
 
+  trace_fontsel(("fontsel_set <%ls>\n", fs->name));
   *(font_spec *) c->data = *fs;   /* structure copy */
 
-  char *boldstr = fs->isbold ? "bold, " : "";
-  char *buf =
+  int boldness = (fs->weight - 1) / 111;
+  static char * boldnesses[] = {
+    "Thin, ",
+    "Extralight, ",
+    "Light, ",
+    "",
+    "Medium, ",
+    "Semibold, ",
+    "Bold, ",
+    "Extrabold, ",
+    "Heavy, "
+  };
+  if (boldness < 0)
+    boldness = 0;
+  else if (boldness >= (int)lengthof(boldnesses))
+    boldness = lengthof(boldnesses) - 1;
+  //char * boldstr = fs->isbold ? "bold, " : "";
+  char * boldstr = boldnesses[boldness];
+#if CYGWIN_VERSION_API_MINOR >= 201
+  int wsize = wcslen(fs->name) + strlen(boldstr) + fs->size ? 31 : 17;
+  wchar * wbuf = newn(wchar, wsize);
+  if (fs->size)
+    swprintf(wbuf, wsize, L"%ls, %s%d%s", fs->name, boldstr, abs(fs->size),
+             fs->size < 0 ? "px" : "pt");
+  else
+    swprintf(wbuf, wsize, L"%ls, %sdefault size", fs->name, boldstr);
+  SetDlgItemTextW(dlg.wnd, c->base_id + 1, wbuf);
+  free(wbuf);
+#else
+  // no swprintf, don't like to fiddle label together for old MinGW
+  char * fn = cs__wcstombs(fs->name);
+  char * buf =
     fs->size
-    ? asform("%s, %s%d-%s", fs->name, boldstr, abs(fs->size),
-             fs->size < 0 ? "pixel" : "point")
-    : asform("%s, %sdefault height", fs->name, boldstr);
+    ? asform("%s, %s%d%s", fn, boldstr, abs(fs->size),
+             fs->size < 0 ? "px" : "pt")
+    : asform("%s, %sdefault size", fn, boldstr);
+  free(fn);
   SetDlgItemText(dlg.wnd, c->base_id + 1, buf);
   free(buf);
+#endif
 }
 
 void
@@ -1220,6 +1338,7 @@ dlg_fontsel_get(control *ctrl, font_spec *fs)
 {
   winctrl *c = ctrl->plat_ctrl;
   assert(c && c->ctrl->type == CTRL_FONTSELECT);
+  trace_fontsel(("fontsel_get <%ls>\n", ((font_spec*)c->data)->name));
   *fs = *(font_spec *) c->data;  /* structure copy */
 }
 
